#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

###OPTIONS################################################

our($opt_help,$opt_verbose,$opt_showbad);
GetOptions("help" => \$opt_help,
	   "v" => \$opt_verbose,
	   "b" => \$opt_showbad);
die "Usage: analyze-choices.pl <datafile> <choicefile> --help --v --b\n"
  unless(scalar(@ARGV)>=2);

##########################################################

our %ssl2map = (
  "RC4-MD5" => "SSL2_RC4_128_MD5",
  "EXP-RC4-MD5" => "SSL2_RC4_128_EXPORT40_MD5",
  "RC2-CBC-MD5" => "SSL2_RC2_128_MD5",
  "EXP-RC2-CBC-MD5" => "SSL2_RC2_128_EXPORT40_MD5",
  "DES-CBC-MD5" => "SSL2_DES_64_MD5",
  "DES-CBC-SHA" => "SSL2_DES_64_SHA",
  "DES-CBC3-MD5" => "SSL2_DES_168_MD5",
  "DES-CBC3-SHA" => "SSL2_DES_168_SHA",
  "RC4-64-MD5" => "SSL2_RC4_64_MD5",
);

our %ssl3map = (
   "EXP-RC4-MD5" => "SSL3_RSA_RC4_40_MD5",
   "RC4-MD5" => "SSL3_RSA_RC4_128_MD5",
   "RC4-SHA" => "SSL3_RSA_RC4_128_SHA",
   "EXP-RC2-CBC-MD5" => "SSL3_RSA_RC2_40_MD5",
   "EXP-DES-CBC-SHA" => "SSL3_RSA_DES_40_SHA",
   "DES-CBC-SHA" => "SSL3_RSA_DES_64_SHA",
   "DES-CBC3-SHA" => "SSL3_RSA_DES_168_SHA",

   "EXP-DH-DSS-DES-CBC-SHA" => "SSL3_DH_DSS_DES_40_SHA",
   "DH-DSS-DES-CBC-SHA" => "SSL3_DH_DSS_DES_64_SHA",
   "DH-DSS-DES-CBC3-SHA" => "SSL3_DH_DSS_DES_168_SHA",
   "EXP-DH-RSA-DES-CBC-SHA" => "SSL3_DH_RSA_DES_40_SHA",
   "DH-RSA-DES-CBC-SHA" => "SSL3_DH_RSA_DES_64_SHA",
   "DH-RSA-DES-CBC3-SHA" => "SSL3_DH_RSA_DES_168_SHA",

   "EXP-EDH-DSS-DES-CBC-SHA" => "SSL3_EDH_DSS_DES_40_SHA",
   "EDH-DSS-DES-CBC-SHA" => "SSL3_EDH_DSS_DES_64_SHA",
   "EDH-DSS-DES-CBC3-SHA" => "SSL3_EDH_DSS_DES_168_SHA",
   "EXP-EDH-RSA-DES-CBC-SHA" => "SSL3_EDH_RSA_DES_40_SHA",
   "EDH-RSA-DES-CBC-SHA" => "SSL3_EDH_RSA_DES_64_SHA",
   "EDH-RSA-DES-CBC3-SHA" => "SSL3_EDH_RSA_DES_168_SHA",

   "EXP-ADH-RC4-MD5" => "SSL3_ADH_RC4_40_MD5",
   "ADH-RC4-MD5" => "SSL3_ADH_RC4_128_MD5",
   "EXP-ADH-DES-CBC-SHA" => "SSL3_ADH_DES_40_SHA",
   "ADH-DES-CBC-SHA" => "SSL3_ADH_DES_64_SHA",
   "ADH-DES-CBC3-SHA" => "SSL3_ADH_DES_168_SHA",

# Homin added these
   "EXP1024-RC4-MD5" => "SSL3_RSA_EXPORT1024_RC4_56_MD5",
   "EXP1024-RC2-CBC-MD5" => "SSL3_RSA_EXPORT1024_RC2_56_MD5",
   "EXP1024-DES-CBC-SHA" => "SSL3_RSA_EXPORT1024_DES_56_SHA",
   "EXP1024-DHE-DSS-DES-CBC-SHA" => "SSL3_DHE_DSS_EXPORT1024_DES_56_SHA",
   "EXP1024-RC4-SHA" => "SSL3_RSA_EXPORT1024_RC4_56_SHA",
   "EXP1024-DHE-DSS-RC4-SHA" => "SSL3_DHE_DSS_EXPORT1024_RC4_56_SHA",
   "DHE-DSS-RC4-SHA" => "SSL3_DHE_DSS_RC4_128_SHA",
   "AES128-SHA" => "SSL3_RSA_AES_128_SHA",
   "DH-DSS-AES128-SHA" => "SSL3_DH_DSS_AES_128_SHA",
   "DH-RSA-AES128-SHA" => "SSL3_DH_RSA_AES_128_SHA",
   "DHE-DSS-AES128-SHA" => "SSL3_DHE_DSS_AES_128_SHA",
   "DHE-RSA-AES128-SHA" => "SSL3_DHE_RSA_AES_128_SHA",
   "ADH-AES128-SHA" => "SSL3_ADH_AES_128_SHA",
   "AES256-SHA" => "SSL3_RSA_AES_256_SHA",
   "DH-DSS-AES256-SHA" => "SSL3_DH_DSS_AES_256_SHA",
   "DH-RSA-AES256-SHA" => "SSL3_DH_RSA_AES_256_SHA",
   "DHE-DSS-AES256-SHA" => "SSL3_DHE_DSS_AES_256_SHA",
   "DHE-RSA-AES256-SHA" => "SSL3_DHE_RSA_AES_256_SHA",
   "ADH-AES256-SHA" => "SSL3_ADH_AES_256_SHA",
);

# * Inconsistency alert:
# * The OpenSSL names of ciphers with ephemeral DH here include the string
# * "DHE", while elsewhere it has always been "EDH".
# * (The alias for the list of all such ciphers also is "EDH".)
# * The specifications speak of "EDH"; maybe we should allow both forms
# * for everything. */

our %tls1map=(
   "EXP1024-RC4-MD5" => "TLS1_RSA_EXPORT1024_RC4_56_MD5",
   "EXP1024-RC2-CBC-MD5" => "TLS1_RSA_EXPORT1024_RC2_56_MD5",
   "EXP1024-DES-CBC-SHA" => "TLS1_RSA_EXPORT1024_DES_56_SHA",
   "EXP1024-DHE-DSS-DES-CBC-SHA" => "TLS1_DHE_DSS_EXPORT1024_DES_56_SHA",
   "EXP1024-RC4-SHA" => "TLS1_RSA_EXPORT1024_RC4_56_SHA",
   "EXP1024-DHE-DSS-RC4-SHA" => "TLS1_DHE_DSS_EXPORT1024_RC4_56_SHA",
   "DHE-DSS-RC4-SHA" => "TLS1_DHE_DSS_RC4_128_SHA",

# AES ciphersuites from RFC3268
   "AES128-SHA" => "TLS1_RSA_AES_128_SHA",
   "DH-DSS-AES128-SHA" => "TLS1_DH_DSS_AES_128_SHA",
   "DH-RSA-AES128-SHA" => "TLS1_DH_RSA_AES_128_SHA",
   "DHE-DSS-AES128-SHA" => "TLS1_DHE_DSS_AES_128_SHA",
   "DHE-RSA-AES128-SHA" => "TLS1_DHE_RSA_AES_128_SHA",
   "ADH-AES128-SHA" => "TLS1_ADH_AES_128_SHA",

   "AES256-SHA" => "TLS1_RSA_AES_256_SHA",
   "DH-DSS-AES256-SHA" => "TLS1_DH_DSS_AES_256_SHA",
   "DH-RSA-AES256-SHA" => "TLS1_DH_RSA_AES_256_SHA",
   "DHE-DSS-AES256-SHA" => "TLS1_DHE_DSS_AES_256_SHA",
   "DHE-RSA-AES256-SHA" => "TLS1_DHE_RSA_AES_256_SHA",
   "ADH-AES256-SHA" => "TLS1_ADH_AES_256_SHA",

# Homin added these
   "EXP-RC4-MD5" => "TLS1_RSA_RC4_40_MD5",
   "RC4-MD5" => "TLS1_RSA_RC4_128_MD5",
   "RC4-SHA" => "TLS1_RSA_RC4_128_SHA",
   "EXP-RC2-CBC-MD5" => "TLS1_RSA_RC2_40_MD5",
   "EXP-DES-CBC-SHA" => "TLS1_RSA_DES_40_SHA",
   "DES-CBC-SHA" => "TLS1_RSA_DES_64_SHA",
   "DES-CBC3-SHA" => "TLS1_RSA_DES_168_SHA",

   "EXP-EDH-DSS-DES-CBC-SHA" => "TLS1_EDH_DSS_DES_40_SHA",
   "EDH-DSS-DES-CBC-SHA" => "TLS1_EDH_DSS_DES_64_SHA",
   "EDH-DSS-DES-CBC3-SHA" => "TLS1_EDH_DSS_DES_168_SHA",
   "EXP-EDH-RSA-DES-CBC-SHA" => "TLS1_EDH_RSA_DES_40_SHA",
   "EDH-RSA-DES-CBC-SHA" => "TLS1_EDH_RSA_DES_64_SHA",
   "EDH-RSA-DES-CBC3-SHA" => "TLS1_EDH_RSA_DES_168_SHA",
);

##########################################################

main(@ARGV);

###MAIN################################################

sub main
{
  my($file,$tlsfile)=@_;
  open(DATA,"$file") or die "Could not open $file\n";
  my %sites=();        #hash of all sites
  while(my $line=<DATA>)
  {
    chomp($line);
    my @fields=split(" -- ",$line);
    my($site,$protocol) = split(" ",$fields[0]);
    my($status,$suite) = split("\: ",$fields[1]);
    ($suite,undef) = split(" ",$suite);
    my $suitemap;
    $suitemap=\%ssl2map if($protocol=~/SSL2/);
    $suitemap=\%ssl3map if($protocol=~/SSL3/);
    $suitemap=\%tls1map if($protocol=~/TLS1/);
    my $fullsuite=$suitemap->{$suite};
    my(undef,$keyex) = split("\: ",$fields[2]);
    my(undef,$auth) = split("\: ",$fields[3]);
    my(undef,$cert) = split("\: ",$fields[4]);

    $auth="DSS" if($auth=~/NULL/);
    $auth="RSA" if($auth=~/FZA/);

    $cert=1024 if($auth =~ /DSS/ && $cert==160);
    $cert=2048 if($auth =~ /DSS/ && $cert==224);
    $cert=3072 if($auth =~ /DSS/ && $cert==256);
    $cert=15360 if($auth =~ /DSS/ && $cert==512);

    #print "$site | $protocol | $suite | $status | $keyex | $auth | $cert\n";
    #check_hash(\%suitehash,$protocol,$suite);

    #count ssl combinations
    $sites{$site}->{protocol}=0 if(!defined($sites{$site}->{protocol}));
    $sites{$site}->{protocol}+=1 if($protocol=~/SSL2/ &&
				    $sites{$site}->{protocol}%2 != 1);
    $sites{$site}->{protocol}+=2 if($protocol=~/SSL3/ &&
				    $sites{$site}->{protocol}%4 != 2 &&
				    $sites{$site}->{protocol}%4 != 3);
    $sites{$site}->{protocol}+=4 if($protocol=~/TLS1/ &&
				    $sites{$site}->{protocol} < 4);
    ###################

    my @ciphers = split("_",$fullsuite);
    my $last="";
    for my $cipher (@ciphers)
    {
      $cipher = "DHE/EDH" if $cipher =~/DHE|EDH/;
      $cipher = $last." ".$cipher if $cipher =~/^\d+$/;
      $last=$cipher;

      #count md5 but no sha
      $sites{$site}->{hash}=0 if(!defined($sites{$site}->{hash}));
      $sites{$site}->{hash}+=1 if($cipher=~/MD5/ &&
				  $sites{$site}->{hash}%2 != 1);
      $sites{$site}->{hash}+=2 if($cipher=~/SHA/ &&
				  $sites{$site}->{hash}%4 != 2 &&
				  $sites{$site}->{hash}%4 != 3);

      if($cipher =~ /^\w+ \d+$/)
      {
	my($c,$n)=split(' ',$cipher);
	if($c =~ /RC2/)
	{
	  $sites{$site}->{rc2}->{max}=$n if(!defined($sites{$site}->{rc2}->{max}) ||
					    $sites{$site}->{rc2}->{max}<$n);
	  $sites{$site}->{rc2}->{min}=$n if(!defined($sites{$site}->{rc2}->{min}) ||
					    $sites{$site}->{rc2}->{min}>$n);
	}
	if($c =~ /RC4/)
	{
	  $sites{$site}->{rc4}->{max}=$n if(!defined($sites{$site}->{rc4}->{max}) ||
					    $sites{$site}->{rc4}->{max}<$n);
	  $sites{$site}->{rc4}->{min}=$n if(!defined($sites{$site}->{rc4}->{min}) ||
					    $sites{$site}->{rc4}->{min}>$n);
	}
	if($c =~ /DES/)
	{
	  $sites{$site}->{des}->{max}=$n if(!defined($sites{$site}->{des}->{max}) ||
					    $sites{$site}->{des}->{max}<$n);
	  $sites{$site}->{des}->{min}=$n if(!defined($sites{$site}->{des}->{min}) ||
					    $sites{$site}->{des}->{min}>$n);
	}
	if($c =~ /AES/)
	{
	  $sites{$site}->{aes}->{stat}=0 if(!defined($sites{$site}->{aes}->{stat}));
	  $sites{$site}->{aes}->{stat}+=1 if($n==128 &&
					     $sites{$site}->{aes}->{stat}%2 != 1);
	  $sites{$site}->{aes}->{stat}+=2 if($n==256 &&
					     $sites{$site}->{aes}->{stat}%4 != 2 &&
					     $sites{$site}->{aes}->{stat}%4 != 3);
          $sites{$site}->{aes}->{max}=$n if(!defined($sites{$site}->{aes}->{max}) ||
					    $sites{$site}->{aes}->{max}<$n);
	  $sites{$site}->{aes}->{min}=$n if(!defined($sites{$site}->{aes}->{min}) ||
					    $sites{$site}->{aes}->{min}>$n);
	}
      }
    }

    if(!defined($sites{$site}->{cert}->{max}) || $cert > $sites{$site}->{cert}->{max})
    {
      $sites{$site}->{cert}->{max}=$cert;
    }
    if(!defined($sites{$site}->{cert}->{min}) || $cert < $sites{$site}->{cert}->{min})
    {
      $sites{$site}->{cert}->{min}=$cert;
    }
    if(!defined($sites{$site}->{cert}->{$cert}))
    {
      $sites{$site}->{cert}->{$cert}=1;
    }

    $sites{$site}->{keyex}=0 if(!defined($sites{$site}->{keyex}));
    $sites{$site}->{keyex}+=1 if($keyex=~/EDH/ &&
				 $sites{$site}->{keyex}%2 != 1);
    $sites{$site}->{keyex}+=2 if($keyex=~/RSA/ &&
				$sites{$site}->{keyex}%4 != 2 &&
				$sites{$site}->{keyex}%4 != 3);

    $sites{$site}->{auth}=0 if(!defined($sites{$site}->{auth}));
    $sites{$site}->{auth}+=1 if($auth=~/DSS/ &&
				 $sites{$site}->{auth}%2 != 1);
    $sites{$site}->{auth}+=2 if($auth=~/RSA/ &&
				$sites{$site}->{auth}%4 != 2 &&
				$sites{$site}->{auth}%4 != 3);
  }
  close(DATA);



  open(TLS,"$tlsfile") or die "Could not open $tlsfile\n";
  my %sslchoice=();
  my %fullsuites=();

  while(my $line=<TLS>)
  {
    chomp($line);
    next if($line=~/^$/);
    my @fields=split(" -- ",$line);
    my($site,$protocol) = split(" ",$fields[0]);
    my($status,$suite) = split("\: ",$fields[1]);

    next unless(exists($sites{$site}));

    ($suite,undef) = split(" ",$suite);
    my $suitemap;
    $suitemap=\%ssl2map if($protocol=~/SSL2/);
    $suitemap=\%ssl3map if($protocol=~/SSL3/);
    $suitemap=\%tls1map if($protocol=~/TLS1/);
    my $fullsuite=$suitemap->{$suite};
    my(undef,$keyex) = split("\: ",$fields[2]);
    my(undef,$auth) = split("\: ",$fields[3]);
    my(undef,$cert) = split("\: ",$fields[4]);
    $auth="DSS" if($auth=~/NULL/);
    $auth="RSA" if($auth=~/FZA/);
    $cert=1024 if($auth =~ /DSS/ && $cert==160);
    $cert=2048 if($auth =~ /DSS/ && $cert==224);
    $cert=3072 if($auth =~ /DSS/ && $cert==256);
    $cert=15360 if($auth =~ /DSS/ && $cert==512);

      #print "$site | $protocol | $suite | $status | $keyex | $auth | $cert\n";

      $sslchoice{$site}->{protocol}=$protocol;
      $sslchoice{$site}->{keyex}=$keyex;
      $sslchoice{$site}->{auth}=$auth;
      $sslchoice{$site}->{cert}=$cert;

    
    $fullsuites{prot}->{$fullsuite}=0 if(!defined($fullsuites{prot}->{$fullsuite}));
    $fullsuites{prot}->{$fullsuite}++;
    my @ciphers = split("_",$fullsuite);
    my $last="";
    for my $cipher (@ciphers)
    {
      $cipher = "DHE/EDH" if $cipher =~/DHE|EDH/;
      $cipher = $last." ".$cipher if $cipher =~/^\d+$/;
      $last=$cipher;
      $sslchoice{$site}->{hash}=$cipher if($cipher=~/MD5/ || $cipher=~/SHA/);
      if($cipher =~ /^\w+ \d+$/)
      {
	my($c,$n)=split(' ',$cipher);
	$sslchoice{$site}->{block}=$cipher;
      }
    }
    my @temp;
    unshift(@temp,pop(@ciphers));
    unshift(@temp,pop(@ciphers));
    unshift(@temp,pop(@ciphers));
    my $partsuite=join("_",@temp);
    $fullsuites{noprot}->{$partsuite}=0
      if(!defined($fullsuites{noprot}->{$partsuite}));
    $fullsuites{noprot}->{$partsuite}++;
  }
  close(TLS);

  if($opt_verbose)
  {
    printverbose(\%sites,\%sslchoice);
  }
  else
  {
    printstats(\%sites,\%sslchoice,\%fullsuites);
  }
}

sub printstats
{
  my($sites,$sslchoice,$fullsuites)=@_;
  my($prot_bad,$prot_good,$prot_only)=(0,0,0,0,0);
  my($cert_bad,$cert_good,$cert_only)=(0,0,0);
  my($hash_bad,$hash_good,$hash_only)=(0,0,0);
  my($block_bad,$block_good,$block_bad_key,$block_bad_aes,$block_bad_both)=(0,0,0,0,0);
  my($rc2,$rc4,$des,$aes)=(0,0,0,0);
  my($insteaddes,$insteadrc2,$insteadrc4)=(0,0,0);
  my %instead=();

  for my $site (sort keys(%$sites))
  {
    next unless(exists($sslchoice->{$site}));

    if($sites->{$site}->{protocol}==1 ||
       $sites->{$site}->{protocol}==2 ||
       $sites->{$site}->{protocol}==4)
    {
      $prot_only++;
    }
    elsif($sites->{$site}->{protocol}>4 ||
	  ($sslchoice->{$site}->{protocol}=~/SSL3/ &&
	   $sites->{$site}->{protocol}==3))
    {
      $prot_good++;
    }
    else
    {
      $prot_bad++;
      #if($opt_showbad)
      #{
	print "$site chose ",$sslchoice->{$site}->{protocol}," out of ";
	print "SSL2 " if($sites->{$site}->{protocol}%2==1);
	print "SSL3 " if($sites->{$site}->{protocol}%4>=2);
	print "TLS1 " if($sites->{$site}->{protocol}>=4);
	print "\n";
      #}
    }
    print STDERR "weird SSL2: $site\n" if($sslchoice->{$site}->{protocol}=~/SSL2/ && 
					  $sites->{$site}->{protocol}%2!=1);
    print STDERR "weird SSL3: $site\n" if($sslchoice->{$site}->{protocol}=~/SSL3/ &&
					  ($sites->{$site}->{protocol}%4!=2 &&
					   $sites->{$site}->{protocol}%4!=3));
    print STDERR "weird TLS: $site\n" if($sslchoice->{$site}->{protocol}=~/TLS/ && 
					 $sites->{$site}->{protocol}<4);


    #cert key size
    if(scalar(keys(%{$sites->{$site}->{cert}}))-2 == 1)
    {
      $cert_only++;
      print STDERR "weird Cert Key: $site\n"
	if($sslchoice->{$site}->{cert}!=$sites->{$site}->{cert}->{max});
    }
    else
    {
      my $choice=$sslchoice->{$site}->{cert};
      my $bad=0;
      for my $cert (sort keys %{$sites->{$site}->{cert}})
      {
	next if($cert=~/max|min/);
	$bad=1 if($choice<$cert);
      }
      if($bad==1)
      {
	$cert_bad++;
	#if($opt_showbad)
	#{
	  print "$site chose ",$sslchoice->{$site}->{cert}," out of ";
	  print "out of ";
	  for my $cert (sort keys %{$sites->{$site}->{cert}}) {
	    next if($cert=~/max|min/);
	    print "$cert ";
	  }
	  print "\n";
	#}
      }
      elsif($bad==0)
      {
	$cert_good++;
      }

      print STDERR "weird Cert Key: $site\n"
	if($sslchoice->{$site}->{cert} > $sites->{$site}->{cert}->{max} ||
	   $sslchoice->{$site}->{cert} < $sites->{$site}->{cert}->{min});
    }

    my $hashchoice= $sslchoice->{$site}->{hash};
    $hash_only++ if($sites->{$site}->{hash}==1 ||
		    $sites->{$site}->{hash}==2);
    $hash_bad++ if($hashchoice=~/MD5/ && $sites->{$site}->{hash}==3);
    $hash_good++ if($hashchoice=~/SHA/ && $sites->{$site}->{hash}==3);

    print STDERR "weird MD5: $site\n"
      if($hashchoice=~/MD5/ && $sites->{$site}->{hash}==2);
    print STDERR "weird SHA: $site\n"
      if($hashchoice=~/SHA/ && $sites->{$site}->{hash}==1);


    my($c,$n)=split(" ",$sslchoice->{$site}->{block});

    print STDERR "weird RC2: $site\n" if($c=~/RC2/ && !defined($sites->{$site}->{rc2}));
    print STDERR "weird RC4: $site\n" if($c=~/RC4/ && !defined($sites->{$site}->{rc4}));
    print STDERR "weird DES: $site\n" if($c=~/DES/ && !defined($sites->{$site}->{des}));
    print STDERR "weird AES: $site\n" if($c=~/AES/ && !defined($sites->{$site}->{aes}));

    my($max,$min)=(0,0);
    if (defined($sites->{$site}->{rc2})) {
      $max=$sites->{$site}->{rc2}->{max}
	if ($max<$sites->{$site}->{rc2}->{max});
      $min=$sites->{$site}->{rc2}->{min}
	if ($min<$sites->{$site}->{rc2}->{min});
    }
    if (defined($sites->{$site}->{rc4})) {
      $max=$sites->{$site}->{rc4}->{max}
	if ($max<$sites->{$site}->{rc4}->{max});
      $min=$sites->{$site}->{rc4}->{min}
	if ($min<$sites->{$site}->{rc4}->{min});
    }
    if (defined($sites->{$site}->{des})) {
      $max=$sites->{$site}->{des}->{max}
	if ($max<$sites->{$site}->{des}->{max});
      $min=$sites->{$site}->{des}->{min}
	if ($min<$sites->{$site}->{des}->{min});
    }
    if (defined($sites->{$site}->{aes})) {
      $max=$sites->{$site}->{aes}->{max}
	if ($max<$sites->{$site}->{aes}->{max});
      $min=$sites->{$site}->{aes}->{min}
	if ($min<$sites->{$site}->{aes}->{min});
    }

    #print "$site $c $n: ";

    if (($n>=$max or ($n==128 && $max==168)) &&
	($c=~/AES/ || !defined($sites->{$site}->{aes})))
    {
      $block_good++;
    }
    else
    {
      #print STDERR "bad block: $site $c $n\n";
      $block_bad++;
      if($c=~/AES/ || !defined($sites->{$site}->{aes}))
      {
	$block_bad_key++;
	$instead{$max}->{$n}=0 if(!defined($instead{$max}->{$n}));
	$instead{$max}->{$n}++;
      }
      elsif(($n==128 && $max==168) or $n>=$max)
      {
	$block_bad_aes++;
	$insteaddes++ if($c=~/DES/);
	$insteadrc2++ if($c=~/RC2/);
	$insteadrc4++ if($c=~/RC4/);
      }
      else
      {
	$block_bad_both++;
	$insteaddes++ if($c=~/DES/);
	$insteadrc2++ if($c=~/RC2/);
	$insteadrc4++ if($c=~/RC4/);
	$instead{$max}->{$n}=0 if(!defined($instead{$max}->{$n}));
	$instead{$max}->{$n}++;
      }
    }

    $rc2++ if($c=~/RC2/);
    $rc4++ if($c=~/RC4/);
    $des++ if($c=~/DES/);
    $aes++ if($c=~/AES/);
  }

  print "\n\n";
  print "Bad protocol chosen: $prot_bad\n";
  print "Good protocol chosen: $prot_good\n";
  print "Only option chosen: $prot_only\n\n";

  print "Bad key size chosen: $cert_bad\n";
  print "Good key size chosen among many: $cert_good\n";
  print "Only possible size chosen: $cert_only\n\n";

  print "MD5 chosen when SHA available: $hash_bad\n";
  print "SHA chosen when MD5 available: $hash_good\n";
  print "Only option chosen: $hash_only\n\n";

  print "Good block cipher chosen from options given: $block_good\n";
  print "Didn't choose AES when available : ",$block_bad_aes+$block_bad_both,"\n";
  print "Didn't choose max key size : ",$block_bad_key+$block_bad_both,"\n";
  print "Did both (intersection): $block_bad_both\n";
  print "Bad block cipher chosen (union of the above two): $block_bad\n\n";
  print "Chose DES instead of AES: $insteaddes\n";
  print "Chose RC2 instead of AES: $insteadrc2\n";
  print "Chose RC4 instead of AES: $insteadrc4\n";
  for my $c (sort keys %instead)
  {
    for my $d (sort keys %{$instead{$c}})
    {
      print "Chose $d instead of $c: ",$instead{$c}->{$d},"\n";
    }
  }

  print"\n";
  print "$rc2 chose RC2\n";
  print "$rc4 chose RC4\n";
  print "$des chose DES\n";
  print "$aes chose AES\n";
  print "\n";

  for my $fullsuite (sort { $fullsuites->{prot}->{$b} <=> $fullsuites->{prot}->{$a}; } keys %{$fullsuites->{prot}})
  {
    print "$fullsuite: ",$fullsuites->{prot}->{$fullsuite},"\n";
  }
  print "\n";

  for my $partsuite (sort { $fullsuites->{noprot}->{$b} <=> $fullsuites->{noprot}->{$a}; } keys %{$fullsuites->{noprot}})
  {
    print "$partsuite: ",$fullsuites->{noprot}->{$partsuite},"\n";
  }
  print "\n";
}

sub printverbose
{
  my($sites,$sslchoice)=@_;
  for my $site (sort keys(%$sites))
  {
    print "$site\t";

    print $sslchoice->{$site}->{protocol}," ";
    print "out of ";
    print "SSL2 " if($sites->{$site}->{protocol}%2==1);
    print "SSL3 " if($sites->{$site}->{protocol}%4>=2);
    print "TLS1 " if($sites->{$site}->{protocol}>=4);
    print "\n";

    print "\t";
    print $sslchoice->{$site}->{keyex}," ";
    print "out of ";
    print "EDH " if($sites->{$site}->{keyex}%2==1);
    print "RSA " if($sites->{$site}->{keyex}>=2);
    print "\n";

    print "\t";
    print $sslchoice->{$site}->{auth}," ";
    print "out of ";
    print "DSS " if($sites->{$site}->{auth}%2==1);
    print "RSA " if($sites->{$site}->{auth}>=2);
    print "\n";

    print "\t";
    print $sslchoice->{$site}->{cert}," ";
    print "out of ";
    for my $cert (sort keys %{$sites->{$site}->{cert}}) {
      next if($cert=~/max|min/);
      print "$cert ";
    }
    print "\n";

    print "\t";
    print $sslchoice->{$site}->{hash}," ";
    print "out of ";
    print "MD5 " if($sites->{$site}->{hash}%2==1);
    print "SHA " if($sites->{$site}->{hash}>=2);
    print "\n";

    print "\t";
    print $sslchoice->{$site}->{block}," ";
    print "out of ";

    print "RC2-",$sites->{$site}->{rc2}->{max}," "
      if (defined($sites->{$site}->{rc2}->{max}));
    print "RC2-",$sites->{$site}->{rc2}->{min}," "
      if (defined($sites->{$site}->{rc2}->{min}) &&
	  $sites->{$site}->{rc2}->{max} !=
	  $sites->{$site}->{rc2}->{min});
    print "RC4-",$sites->{$site}->{rc4}->{max}," "
      if (defined($sites->{$site}->{rc4}->{max}));
    print "RC4-",$sites->{$site}->{rc4}->{min}," "
      if (defined($sites->{$site}->{rc4}->{min}) &&
	  $sites->{$site}->{rc4}->{max} !=
	  $sites->{$site}->{rc4}->{min});
    print "DES-",$sites->{$site}->{des}->{max}," "
      if (defined($sites->{$site}->{des}->{max}));
    print "DES-",$sites->{$site}->{des}->{min}," "
      if (defined($sites->{$site}->{des}->{min}) &&
	  $sites->{$site}->{des}->{max} !=
	  $sites->{$site}->{des}->{min});
    print "AES-",$sites->{$site}->{aes}->{max}," "
      if (defined($sites->{$site}->{aes}->{max}));
    print "AES-",$sites->{$site}->{aes}->{min}," "
      if (defined($sites->{$site}->{aes}->{min}) &&
	  $sites->{$site}->{aes}->{max} !=
	  $sites->{$site}->{aes}->{min});
    print "\n";
  }

}

sub check_hash
{
  my($hash,$protocol,$suite)=@_;
  if(not defined $hash->{$protocol}->{$suite})
  {
    $hash->{$protocol}->{$suite}=0;
  }
}
