#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

###OPTIONS################################################

our($opt_help,$opt_good,$opt_verbose);
GetOptions("help" => \$opt_help, "good" => \$opt_good,
	   "v" => \$opt_verbose);
die "Usage: stats.pl <datafile> --help --good --v\n"
  unless(scalar(@ARGV)>=1);

##########################################################

our %ssl2map = (
  "RC4-MD5" => "SSL2_RC4_128_MD5",
  "EXP-RC4-MD5" => "SSL2_RC4_128_EXPORT40_MD5",
  "RC2-CBC-MD5" => "SSL2_RC2_128_MD5",
  "EXP-RC2-CBC-MD5" => "SSL2_RC2_128_EXPORT40_MD5",
  "DES-CBC-MD5" => "SSL2_DES_64_MD5",
  "DES-CBC-SHA" => "SSL2_DES_64_SHA",
  "DES-CBC3-MD5" => "SSL2_DES_168_MD5",
  "DES-CBC3-SHA" => "SSL2_DES_168_SHA",
  "RC4-64-MD5" => "SSL2_RC4_64_MD5",
);

our %ssl3map = (
   "EXP-RC4-MD5" => "SSL3_RSA_RC4_40_MD5",
   "RC4-MD5" => "SSL3_RSA_RC4_128_MD5",
   "RC4-SHA" => "SSL3_RSA_RC4_128_SHA",
   "EXP-RC2-CBC-MD5" => "SSL3_RSA_RC2_40_MD5",
   "EXP-DES-CBC-SHA" => "SSL3_RSA_DES_40_SHA",
   "DES-CBC-SHA" => "SSL3_RSA_DES_64_SHA",
   "DES-CBC3-SHA" => "SSL3_RSA_DES_168_SHA",

   "EXP-DH-DSS-DES-CBC-SHA" => "SSL3_DH_DSS_DES_40_SHA",
   "DH-DSS-DES-CBC-SHA" => "SSL3_DH_DSS_DES_64_SHA",
   "DH-DSS-DES-CBC3-SHA" => "SSL3_DH_DSS_DES_168_SHA",
   "EXP-DH-RSA-DES-CBC-SHA" => "SSL3_DH_RSA_DES_40_SHA",
   "DH-RSA-DES-CBC-SHA" => "SSL3_DH_RSA_DES_64_SHA",
   "DH-RSA-DES-CBC3-SHA" => "SSL3_DH_RSA_DES_168_SHA",

   "EXP-EDH-DSS-DES-CBC-SHA" => "SSL3_EDH_DSS_DES_40_SHA",
   "EDH-DSS-DES-CBC-SHA" => "SSL3_EDH_DSS_DES_64_SHA",
   "EDH-DSS-DES-CBC3-SHA" => "SSL3_EDH_DSS_DES_168_SHA",
   "EXP-EDH-RSA-DES-CBC-SHA" => "SSL3_EDH_RSA_DES_40_SHA",
   "EDH-RSA-DES-CBC-SHA" => "SSL3_EDH_RSA_DES_64_SHA",
   "EDH-RSA-DES-CBC3-SHA" => "SSL3_EDH_RSA_DES_168_SHA",

   "EXP-ADH-RC4-MD5" => "SSL3_ADH_RC4_40_MD5",
   "ADH-RC4-MD5" => "SSL3_ADH_RC4_128_MD5",
   "EXP-ADH-DES-CBC-SHA" => "SSL3_ADH_DES_40_SHA",
   "ADH-DES-CBC-SHA" => "SSL3_ADH_DES_64_SHA",
   "ADH-DES-CBC3-SHA" => "SSL3_ADH_DES_168_SHA",

# Homin added these
   "EXP1024-RC4-MD5" => "SSL3_RSA_EXPORT1024_RC4_56_MD5",
   "EXP1024-RC2-CBC-MD5" => "SSL3_RSA_EXPORT1024_RC2_56_MD5",
   "EXP1024-DES-CBC-SHA" => "SSL3_RSA_EXPORT1024_DES_56_SHA",
   "EXP1024-DHE-DSS-DES-CBC-SHA" => "SSL3_DHE_DSS_EXPORT1024_DES_56_SHA",
   "EXP1024-RC4-SHA" => "SSL3_RSA_EXPORT1024_RC4_56_SHA",
   "EXP1024-DHE-DSS-RC4-SHA" => "SSL3_DHE_DSS_EXPORT1024_RC4_56_SHA",
   "DHE-DSS-RC4-SHA" => "SSL3_DHE_DSS_RC4_128_SHA",
   "AES128-SHA" => "SSL3_RSA_AES_128_SHA",
   "DH-DSS-AES128-SHA" => "SSL3_DH_DSS_AES_128_SHA",
   "DH-RSA-AES128-SHA" => "SSL3_DH_RSA_AES_128_SHA",
   "DHE-DSS-AES128-SHA" => "SSL3_DHE_DSS_AES_128_SHA",
   "DHE-RSA-AES128-SHA" => "SSL3_DHE_RSA_AES_128_SHA",
   "ADH-AES128-SHA" => "SSL3_ADH_AES_128_SHA",
   "AES256-SHA" => "SSL3_RSA_AES_256_SHA",
   "DH-DSS-AES256-SHA" => "SSL3_DH_DSS_AES_256_SHA",
   "DH-RSA-AES256-SHA" => "SSL3_DH_RSA_AES_256_SHA",
   "DHE-DSS-AES256-SHA" => "SSL3_DHE_DSS_AES_256_SHA",
   "DHE-RSA-AES256-SHA" => "SSL3_DHE_RSA_AES_256_SHA",
   "ADH-AES256-SHA" => "SSL3_ADH_AES_256_SHA",
);

# * Inconsistency alert:
# * The OpenSSL names of ciphers with ephemeral DH here include the string
# * "DHE", while elsewhere it has always been "EDH".
# * (The alias for the list of all such ciphers also is "EDH".)
# * The specifications speak of "EDH"; maybe we should allow both forms
# * for everything. */

our %tls1map=(
   "EXP1024-RC4-MD5" => "TLS1_RSA_EXPORT1024_RC4_56_MD5",
   "EXP1024-RC2-CBC-MD5" => "TLS1_RSA_EXPORT1024_RC2_56_MD5",
   "EXP1024-DES-CBC-SHA" => "TLS1_RSA_EXPORT1024_DES_56_SHA",
   "EXP1024-DHE-DSS-DES-CBC-SHA" => "TLS1_DHE_DSS_EXPORT1024_DES_56_SHA",
   "EXP1024-RC4-SHA" => "TLS1_RSA_EXPORT1024_RC4_56_SHA",
   "EXP1024-DHE-DSS-RC4-SHA" => "TLS1_DHE_DSS_EXPORT1024_RC4_56_SHA",
   "DHE-DSS-RC4-SHA" => "TLS1_DHE_DSS_RC4_128_SHA",

# AES ciphersuites from RFC3268
   "AES128-SHA" => "TLS1_RSA_AES_128_SHA",
   "DH-DSS-AES128-SHA" => "TLS1_DH_DSS_AES_128_SHA",
   "DH-RSA-AES128-SHA" => "TLS1_DH_RSA_AES_128_SHA",
   "DHE-DSS-AES128-SHA" => "TLS1_DHE_DSS_AES_128_SHA",
   "DHE-RSA-AES128-SHA" => "TLS1_DHE_RSA_AES_128_SHA",
   "ADH-AES128-SHA" => "TLS1_ADH_AES_128_SHA",

   "AES256-SHA" => "TLS1_RSA_AES_256_SHA",
   "DH-DSS-AES256-SHA" => "TLS1_DH_DSS_AES_256_SHA",
   "DH-RSA-AES256-SHA" => "TLS1_DH_RSA_AES_256_SHA",
   "DHE-DSS-AES256-SHA" => "TLS1_DHE_DSS_AES_256_SHA",
   "DHE-RSA-AES256-SHA" => "TLS1_DHE_RSA_AES_256_SHA",
   "ADH-AES256-SHA" => "TLS1_ADH_AES_256_SHA",

# Homin added these
   "EXP-RC4-MD5" => "TLS1_RSA_RC4_40_MD5",
   "RC4-MD5" => "TLS1_RSA_RC4_128_MD5",
   "RC4-SHA" => "TLS1_RSA_RC4_128_SHA",
   "EXP-RC2-CBC-MD5" => "TLS1_RSA_RC2_40_MD5",
   "EXP-DES-CBC-SHA" => "TLS1_RSA_DES_40_SHA",
   "DES-CBC-SHA" => "TLS1_RSA_DES_64_SHA",
   "DES-CBC3-SHA" => "TLS1_RSA_DES_168_SHA",

   "EXP-EDH-DSS-DES-CBC-SHA" => "TLS1_EDH_DSS_DES_40_SHA",
   "EDH-DSS-DES-CBC-SHA" => "TLS1_EDH_DSS_DES_64_SHA",
   "EDH-DSS-DES-CBC3-SHA" => "TLS1_EDH_DSS_DES_168_SHA",
   "EXP-EDH-RSA-DES-CBC-SHA" => "TLS1_EDH_RSA_DES_40_SHA",
   "EDH-RSA-DES-CBC-SHA" => "TLS1_EDH_RSA_DES_64_SHA",
   "EDH-RSA-DES-CBC3-SHA" => "TLS1_EDH_RSA_DES_168_SHA",
);

##########################################################

main(@ARGV);

###MAIN################################################

sub main
{
  my($file)=@_;
  print "$file\n";
  open(DATA,"$file") or die "Could not open $file\n";
  my %suitehash=();    #number of sites that support each suite
  my %sites=();        #hash of all sites
  my %cipher_hash=();  #number of sites that support each cipher
  my %etal=();         #number of sites that support each ke, auth, certsize
  while(my $line=<DATA>)
  {
    chomp($line);
    my @fields=split(" -- ",$line);
    my($site,$protocol) = split(" ",$fields[0]);
    my($status,$suite) = split("\: ",$fields[1]);
    ($suite,undef) = split(" ",$suite);
    my $suitemap;
    $suitemap=\%ssl2map if($protocol=~/SSL2/);
    $suitemap=\%ssl3map if($protocol=~/SSL3/);
    $suitemap=\%tls1map if($protocol=~/TLS1/);
    my $fullsuite=$suitemap->{$suite};
    my(undef,$keyex) = split("\: ",$fields[2]);
    my(undef,$auth) = split("\: ",$fields[3]);
    my(undef,$cert) = split("\: ",$fields[4]);

    $auth="DSS" if($auth=~/NULL/);
    $auth="RSA" if($auth=~/FZA/);

    $cert=1024 if($auth =~ /DSS/ && $cert==160);
    $cert=2048 if($auth =~ /DSS/ && $cert==224);
    $cert=3072 if($auth =~ /DSS/ && $cert==256);
    $cert=15360 if($auth =~ /DSS/ && $cert==512);

    #print "$site | $protocol | $suite | $status | $keyex | $auth | $cert\n";
    #check_hash(\%suitehash,$protocol,$suite);

    $suitehash{$protocol}->{"$fullsuite\t$keyex\t$auth\t$cert"}++;

    #count ssl combinations
    $sites{$site}->{protocol}=0 if(!defined($sites{$site}->{protocol}));
    $sites{$site}->{protocol}+=1 if($protocol=~/SSL2/ &&
				    $sites{$site}->{protocol}%2 != 1);
    $sites{$site}->{protocol}+=2 if($protocol=~/SSL3/ &&
				    $sites{$site}->{protocol}%4 != 2 &&
				    $sites{$site}->{protocol}%4 != 3);
    $sites{$site}->{protocol}+=4 if($protocol=~/TLS1/ &&
				    $sites{$site}->{protocol} < 4);
    ###################

    $cipher_hash{$protocol}->{$site}=1;

    my @ciphers = split("_",$fullsuite);
    my $last="";
    for my $cipher (@ciphers)
    {
      #next if $cipher =~ /TXT/;
      #next if $cipher =~ /WITH/;
      $cipher = "DHE/EDH" if $cipher =~/DHE|EDH/;
      $cipher = $last." ".$cipher if $cipher =~/^\d+$/;
      $cipher_hash{$cipher}->{$site}=1;
      $last=$cipher;

      #count md5 but no sha
      $sites{$site}->{hash}=0 if(!defined($sites{$site}->{hash}));
      $sites{$site}->{hash}+=1 if($cipher=~/MD5/ &&
				  $sites{$site}->{hash}%2 != 1);
      $sites{$site}->{hash}+=2 if($cipher=~/SHA/ &&
				  $sites{$site}->{hash}%4 != 2 &&
				  $sites{$site}->{hash}%4 != 3);

      if($cipher =~ /^\w+ \d+$/)
      {
	my($c,$n)=split(' ',$cipher);
	if($c =~ /RC2/)
	{
	  $sites{$site}->{rc2}->{max}=$n if(!defined($sites{$site}->{rc2}->{max}) ||
					    $sites{$site}->{rc2}->{max}<$n);
	  $sites{$site}->{rc2}->{min}=$n if(!defined($sites{$site}->{rc2}->{min}) ||
					    $sites{$site}->{rc2}->{min}>$n);
	}
	if($c =~ /RC4/)
	{
	  $sites{$site}->{rc4}->{max}=$n if(!defined($sites{$site}->{rc4}->{max}) ||
					    $sites{$site}->{rc4}->{max}<$n);
	  $sites{$site}->{rc4}->{min}=$n if(!defined($sites{$site}->{rc4}->{min}) ||
					    $sites{$site}->{rc4}->{min}>$n);
	}
	if($c =~ /DES/)
	{
	  $sites{$site}->{des}->{max}=$n if(!defined($sites{$site}->{des}->{max}) ||
					    $sites{$site}->{des}->{max}<$n);
	  $sites{$site}->{des}->{min}=$n if(!defined($sites{$site}->{des}->{min}) ||
					    $sites{$site}->{des}->{min}>$n);
	}
	if($c =~ /AES/)
	{
	  $sites{$site}->{aes}->{stat}=0 if(!defined($sites{$site}->{aes}->{stat}));
	  $sites{$site}->{aes}->{stat}+=1 if($n==128 &&
					     $sites{$site}->{aes}->{stat}%2 != 1);
	  $sites{$site}->{aes}->{stat}+=2 if($n==256 &&
					     $sites{$site}->{aes}->{stat}%4 != 2 &&
					     $sites{$site}->{aes}->{stat}%4 != 3);
          $sites{$site}->{aes}->{max}=$n if(!defined($sites{$site}->{aes}->{max}) ||
					    $sites{$site}->{aes}->{max}<$n);
	  $sites{$site}->{aes}->{min}=$n if(!defined($sites{$site}->{aes}->{min}) ||
					    $sites{$site}->{aes}->{min}>$n);
	}
      }
    }

    $etal{keyex}->{$keyex}->{$site}=1;
    $etal{auth}->{$auth}->{$site}=1;
    $etal{cert}->{$cert}->{$site}=1;
    $etal{keyexauth}->{$keyex.' '.$auth}->{$site}=1;

    if(!defined($sites{$site}->{cert}->{max}) || $cert > $sites{$site}->{cert}->{max})
    {
      $sites{$site}->{cert}->{max}=$cert;
    }
    if(!defined($sites{$site}->{cert}->{min}) || $cert < $sites{$site}->{cert}->{min})
    {
      $sites{$site}->{cert}->{min}=$cert;
    }
    if(!defined($sites{$site}->{cert}->{$cert}))
    {
      $sites{$site}->{cert}->{$cert}=1;
    }

    $sites{$site}->{keyex}=0 if(!defined($sites{$site}->{keyex}));
    $sites{$site}->{keyex}+=1 if($keyex=~/EDH/ &&
				 $sites{$site}->{keyex}%2 != 1);
    $sites{$site}->{keyex}+=2 if($keyex=~/RSA/ &&
				$sites{$site}->{keyex}%4 != 2 &&
				$sites{$site}->{keyex}%4 != 3);

    $sites{$site}->{auth}=0 if(!defined($sites{$site}->{auth}));
    $sites{$site}->{auth}+=1 if($auth=~/DSS/ &&
				 $sites{$site}->{auth}%2 != 1);
    $sites{$site}->{auth}+=2 if($auth=~/RSA/ &&
				$sites{$site}->{auth}%4 != 2 &&
				$sites{$site}->{auth}%4 != 3);
  }
  close(DATA);

  if($opt_good)
  {
    for my $site (keys(%sites))
    {
      print "$site\n" if($sites{$site}>0);
    }
  }
  else
  {
    my $total=scalar(keys(%sites));
    print "SSL sites (supports at least one suite): $total\n\n";


    if($opt_verbose)
    {
      #print suite support totals
      print "protocol\tsuite\tkey exchange\tauthentication\tcert. key size\tsupported\tpercentage\n";
      for my $protocol(sort keys(%suitehash))
      {
	for my $suite(sort keys(%{$suitehash{$protocol}}))
	{
	  my $num=$suitehash{$protocol}->{$suite};
	printf("%s\t%s\t%d\t%.2f %s\n",$protocol,$suite,$num,100*$num/$total,"%");
	}
      }
      print "\n";
    }

    #print cipher support totals
    print "Support among SSL sites\n";
    for my $cipher (sort keys %cipher_hash)
    {
      my $num = scalar(keys %{$cipher_hash{$cipher}});
      printf("%s\t%d\t%.2f %s\n",$cipher,$num,100*$num/$total,"%");
    }
    print "\n";

    my $count=0;

    my($type1,$type2,$type3,$type4,$type5,$type6,$type7)=
      (0,0,0,0,0,0,0);
    my %rc2maxhash=();
    my %rc4maxhash=();
    my %desmaxhash=();
    my %aesmaxhash=();
    my %certmaxhash=();
    my %strongsites=();
    my %weaksites=();
    my @aeslist=(0,0,0,0);
    my $multcert=0;
    my($weak,$semi,$strong)=(0,0,0);
    my($murrayweak,$murraystrong,$murrayweakcipher)=(0,0,0);
    for my $site (keys(%sites))
    {
      my($maybeweak,$maybesemi,$maybestrong,$ourcipher)=(0,1,1,0);
      my($murraymaybeweak,$murraymaybestrong,
	 $murraycipher)=(0,1,0);
      $type1++ if($sites{$site}->{protocol}==1);
      $type2++ if($sites{$site}->{protocol}==2);
      $type3++ if($sites{$site}->{protocol}==3);
      $type4++ if($sites{$site}->{protocol}==4);
      $type5++ if($sites{$site}->{protocol}==5);
      $type6++ if($sites{$site}->{protocol}==6);
      $type7++ if($sites{$site}->{protocol}==7);
      if(defined($sites{$site}->{rc2}))
      {
	my $rc2max=$sites{$site}->{rc2}->{max};
	my $rc2min=$sites{$site}->{rc2}->{min};
	$rc2maxhash{$rc2max}=0
	  if(!defined($rc2maxhash{$rc2max}));
	$rc2maxhash{$rc2max}++;
	$maybestrong=0 if($rc2min<128);
	$maybesemi=0 if($rc2min<128);
	$ourcipher=$rc2max if($rc2max>$ourcipher);
	$murraycipher=$rc2max if($rc2max>$murraycipher);
      }
      if(defined($sites{$site}->{rc4}))
      {
	my $rc4max=$sites{$site}->{rc4}->{max};
	my $rc4min=$sites{$site}->{rc4}->{min};
	$rc4maxhash{$rc4max}=0
	  if(!defined($rc4maxhash{$rc4max}));
	$rc4maxhash{$rc4max}++;
	$maybestrong=0 if($rc4min<128);
	$maybesemi=0 if($rc4min<128);
	$ourcipher=$rc4max if($rc4max>$ourcipher);
	$murraycipher=$rc4max if($rc4max>$murraycipher);
      }
      if(defined($sites{$site}->{des}))
      {
	my $desmax=$sites{$site}->{des}->{max};
	my $desmin=$sites{$site}->{des}->{min};
	$desmaxhash{$desmax}=0
	  if(!defined($desmaxhash{$desmax}));
	$desmaxhash{$desmax}++;
	$maybestrong=0 if($desmin<128);
	$maybesemi=0 if($desmin<128);
	$ourcipher=$desmax if($desmax>$ourcipher);
	$murraycipher=$desmax if($desmax>$murraycipher);
      }
      if(defined($sites{$site}->{aes}))
      {
	my $aesmax=$sites{$site}->{aes}->{max};
	my $aesmin=$sites{$site}->{aes}->{min};
	$aesmaxhash{$aesmax}=0
	  if(!defined($aesmaxhash{$aesmax}));
	$aesmaxhash{$aesmax}++;
	$maybestrong=0 if($aesmin<128);
	$maybesemi=0 if($aesmin<128);
	$ourcipher=$aesmax if($aesmax>$ourcipher);
	$murraycipher=$aesmax if($aesmax>$murraycipher);
	my $aesnum=$sites{$site}->{aes}->{stat};
	$aeslist[$aesnum]++;
      }
      if(defined($sites{$site}->{cert}))
      {
	my $certmax=$sites{$site}->{cert}->{max};
	my $certmin=$sites{$site}->{cert}->{min};
	$multcert++ if(scalar(keys(%{$sites{$site}->{cert}}))-2 > 1);
	#print STDERR "multcert: $site\n" if(scalar(keys(%{$sites{$site}->{cert}}))-2 > 1);
	print "bad cert: $site\n" if($certmax<=512 && $opt_verbose);
	$certmaxhash{$certmax}=0
	  if(!defined($certmaxhash{$certmax}));
	$certmaxhash{$certmax}++;
	$maybeweak=1 if($certmax<1024);
	$maybestrong=0 if($certmin<1024);
	$maybesemi=0 if($certmin<1024);
	$murraymaybeweak=1 if($certmax<=512);
	$murraymaybestrong=0 if($certmax<1024);
      }
      $maybeweak=1 if($ourcipher<128);
      $murraymaybeweak=1 if($murraycipher<=40);
      $murrayweakcipher++ if($murraycipher<=40);
      $murraymaybestrong=0 if($murraycipher<64);
      $maybeweak=1 if($sites{$site}->{protocol}==1);
      $maybestrong=0 if($sites{$site}->{protocol}==1 || 
			$sites{$site}->{protocol}==3 || 
			$sites{$site}->{protocol}==5 || 
			$sites{$site}->{protocol}==7);
      $maybesemi=0 if($sites{$site}->{protocol}==1 || 
		      $sites{$site}->{protocol}==3 || 
		      $sites{$site}->{protocol}==5 || 
		      $sites{$site}->{protocol}==7);
      $murraymaybeweak=1 if($sites{$site}->{protocol}==1);
      $murraymaybestrong=0 if($sites{$site}->{protocol}==1);
      $maybeweak=1 if($sites{$site}->{hash}==1);
      $maybestrong=0 if($sites{$site}->{hash}!=2);
      $weak++ if($maybeweak==1);
      $strong++ if($maybestrong==1);
      $semi++ if($maybesemi==1);
      $murrayweak++ if($murraymaybeweak==1);
      $murraystrong++ if($murraymaybestrong==1);
      $strongsites{$site}=1 if($maybestrong==1);
      $weaksites{$site}=1 if($maybeweak==1);
    }

    print "Maximum RC2 Key Size:\n";
    for my $rc2size (sort {$a <=>$b} keys %rc2maxhash)
    {
      my $ct = $rc2maxhash{$rc2size};
      printf("%d\t%d\t%.2f %s\n",$rc2size,$ct,100*$ct/$total,"%");
    }
    print "\n";


    print "Maximum RC4 Key Size:\n";
    for my $rc4size (sort {$a <=>$b} keys %rc4maxhash)
    {
      my $ct = $rc4maxhash{$rc4size};      
      printf("%d\t%d\t%.2f %s\n",$rc4size,$ct,100*$ct/$total,"%");
    }
    print "\n";

    print "Maximum DES Key Size:\n";
    for my $dessize (sort {$a <=>$b} keys %desmaxhash)
    {
      my $ct = $desmaxhash{$dessize};      
      printf("%d\t%d\t%.2f %s\n",$dessize,$ct,100*$ct/$total,"%");
    }
    print "\n";

    print "Maximum AES Key Size:\n";
    for my $aessize (sort {$a <=>$b} keys %aesmaxhash)
    {
      my $ct = $aesmaxhash{$aessize};      
      printf("%d\t%d\t%.2f %s\n",$aessize,$ct,100*$ct/$total,"%");
    }
    print "\n";

    print "AES Types\n";
    print "AES128 AES256 Number Percentage\n";
    my $aesnum=$aeslist[0];
    for my $riure (@aeslist)
    {
      $aesnum+=$riure;
    }
    $aesnum=$total-$aesnum;
    print "No & No & ";
    printf("%d & %.2f%s\n",$aesnum,100*$aesnum/$total,"\\%");
    $aesnum=$aeslist[1];
    print "No & Yes & ";
    printf("%d & %.2f%s\n",$aesnum,100*$aesnum/$total,"\\%");
    $aesnum=$aeslist[2];
    print "Yes & No & ";
    printf("%d & %.2f%s\n",$aesnum,100*$aesnum/$total,"\\%");
    $aesnum=$aeslist[3];
    print "Yes & Yes & ";
    printf("%d & %.2f%s\n",$aesnum,100*$aesnum/$total,"\\%");
    print "\n";

    #print etal totals
    print "Key Exchange\n";
    for my $cipher (sort keys %{$etal{keyex}})
    {
      my $num = scalar(keys %{$etal{keyex}->{$cipher}});
      printf("%s\t%d\t%.2f %s\n",$cipher,$num,100*$num/$total,"%");
    }
    print "\n";

    print "Authentication\n";
    for my $cipher (sort keys %{$etal{auth}})
    {
      my $num = scalar(keys %{$etal{auth}->{$cipher}});
      printf("%s\t%d\t%.2f %s\n",$cipher,$num,100*$num/$total,"%");
    }
    print "\n";

    print "Key Exchange + Authentication\n";
    for my $cipher (sort keys %{$etal{keyexauth}})
    {
      my $num = scalar(keys %{$etal{keyexauth}->{$cipher}});
      printf("%s\t%d\t%.2f %s\n",$cipher,$num,100*$num/$total,"%");
    }
    print "\n";

    print "Certificate Key Size Support:\n";
    for my $cipher (sort {$a <=>$b} keys %{$etal{cert}})
    {
      my $num = scalar(keys %{$etal{cert}->{$cipher}});
      printf("%s\t%d\t%.2f %s\n",$cipher,$num,100*$num/$total,"%");
    }
    print "\n";

    print "Maximum Certificate Key Size:\n";
    for my $certsize (sort {$a <=>$b} keys %certmaxhash)
    {
      my $ct = $certmaxhash{$certsize};      
      printf("%d\t%d\t%.2f %s\n",$certsize,$ct,100*$ct/$total,"%");
    }
    print "\n";

    print "Servers with multiple cert key sizes: $multcert\n\n";

    print "SSL Types\n";
    print "SSL2 SSL3 TLS1\n";
    print "Yes & No & No & $type1 & ";
    printf("%.2f %s\n",100*$type1/$total,"\\% \\\\");
    print "No & Yes & No & $type2 & ";
    printf("%.2f %s\n",100*$type2/$total,"\\% \\\\");
    print "Yes & Yes & No & $type3 & ";
    printf("%.2f %s\n",100*$type3/$total,"\\% \\\\");
    print "No & No & Yes & $type4 & ";
    printf("%.2f %s\n",100*$type4/$total,"\\% \\\\");
    print "Yes & No & Yes & $type5 & ";
    printf("%.2f %s\n",100*$type5/$total,"\\% \\\\");
    print "No & Yes & Yes & $type6 & ";
    printf("%.2f %s\n",100*$type6/$total,"\\% \\\\");
    print "Yes & Yes & Yes & $type7 & ";
    printf("%.2f %s\n",100*$type7/$total,"\\% \\\\");
    print "Total: ",$type1+$type2+$type3+$type4+$type5+$type6+$type7,"\n";
    print "\n";

    printf("Weak servers: %d %.2f\n",$weak,100*$weak/$total);
    printf("Medium servers: %d %.2f\n",$total-$weak-$strong,
	   100*($total-$weak-$strong)/$total);
    #printf("Semi-Strong servers: %d %.2f\n",$semi,100*$semi/$total);
    printf("Strong servers: %d %.2f\n",$strong,100*$strong/$total);
    printf("Murray Weak servers: %d %.2f\n",$murrayweak,
	   100*$murrayweak/$total);
    printf("Murray Medium servers: %d %.2f\n",
	   $total-$murrayweak-$murraystrong,
	   100*($total-$murrayweak-$murraystrong)/$total);
    printf("Murray Strong servers: %d %.2f\n",$murraystrong,
	   100*$murraystrong/$total);
    printf("Murray Weak Cipher servers: %d %.2f\n",$murrayweakcipher,
	   100*$murrayweakcipher/$total);
    print "\n";

    if($opt_verbose)
    {
      print "SSL2 only sites:\n";
      $count=0;
      for my $site (keys(%sites))
      {
	if($sites{$site}->{protocol}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";

      print "MD5 only sites:\n";
      $count=0;
      for my $site (keys(%sites))
      {
	if($sites{$site}->{hash}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";
      
      print "EDH only sites:\n";
      $count=0;
      for my $site (keys(%sites))
      {
	if($sites{$site}->{keyex}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";

      print "DSS only sites:\n";
      $count=0;
      for my $site (keys(%sites))
      {
	if($sites{$site}->{auth}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";
       
      print "Strong sites:\n";
      $count=0;
      for my $site (keys(%strongsites))
      {
	if($strongsites{$site}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";

      print "Weak sites:\n";
      $count=0;
      for my $site (keys(%weaksites))
      {
	if($weaksites{$site}==1)
	{
	  print "$site\n";
	  $count++;
	}
      }
      print "$count\n";
      print "\n";
   }
  }
}

sub check_hash
{
  my($hash,$protocol,$suite)=@_;
  if(not defined $hash->{$protocol}->{$suite})
  {
    $hash->{$protocol}->{$suite}=0;
  }
}
